import {
    Body,
    Controller,
    Get,
    HttpCode,
    HttpStatus,
    Post,
    UploadedFiles,
    UseGuards,
    UseInterceptors
} from "@nestjs/common";
import { AdminAuthGuard } from "src/core/guards/admin-auth.guard";
import { Constants } from "src/utils/constants";
import { FileFieldsInterceptor } from "@nestjs/platform-express";
import { UploadedMulterFileI } from "src/modules/SpacesModule/SpacesService";
import { ArticlePageCmsService } from "./article-page-cms.service";
import { UpdateArticlePageCmsDto } from "./dto/update-article-page-cms.dto";

@Controller({ path: "article-page-cms", version: Constants.API_VERSION_1 })

export class ArticlePageCmsController {
    constructor(private readonly service: ArticlePageCmsService) {
    }

    @Post()
    @HttpCode(HttpStatus.OK)
    @UseGuards(AdminAuthGuard)
    @UseInterceptors(FileFieldsInterceptor(
      [
          { name: "bannerImage", maxCount: 1 }
      ]
    ))
    updateOrCreateData(
      @Body() dto: UpdateArticlePageCmsDto,
      @UploadedFiles() files: {
          bannerImage: UploadedMulterFileI
      }
    ) {
        return this.service.updateOrCreateData(dto, files);
    }

    @Get()
    // @UseGuards(AdminAuthGuard)
    findAll() {
        return this.service.findAll();
    }
}
