import {
    Body,
    Controller,
    Get,
    HttpCode,
    HttpStatus,
    Post,
    UploadedFiles,
    UseGuards,
    UseInterceptors
} from "@nestjs/common";
import { AdminAuthGuard } from "src/core/guards/admin-auth.guard";
import { Constants } from "src/utils/constants";
import { FileFieldsInterceptor } from "@nestjs/platform-express";
import { UploadedMulterFileI } from "src/modules/SpacesModule/SpacesService";
import { CartCmsService } from "./cart-cms.service";
import { UpdateCartCmsDto } from "./dto/update-cart-cms.dto";

@Controller({ path: "cart-cms", version: Constants.API_VERSION_1 })

export class CartCmsController {
    constructor(private readonly service: CartCmsService) {
    }

    @Post()
    @HttpCode(HttpStatus.OK)
    @UseGuards(AdminAuthGuard)
    @UseInterceptors(FileFieldsInterceptor(
      [
          { name: "topBannerImage", maxCount: 1 },
          { name: "bottomBannerImage", maxCount: 1 }
      ]
    ))
    updateOrCreateData(
      @Body() dto: UpdateCartCmsDto,
      @UploadedFiles() files: {
          topBannerImage: UploadedMulterFileI,
          bottomBannerImage: UploadedMulterFileI
      }
    ) {
        return this.service.updateOrCreateData(dto, files);
    }

    @Get()
    // @UseGuards(AdminAuthGuard)
    findAll() {
        return this.service.findAll();
    }
}
