import { EntityRepository, getConnection, Repository } from "typeorm";
import { UpdateHomePageSubNavDto } from "./dto/update-home-page-sub-nav.dto";
import { HomePageSubNavEntity } from "./entities/home-page-sub-nav.entity";
import { BaseHomePageSubNavDto } from "./dto/base-home-page-sub-nav.dto";

@EntityRepository(HomePageSubNavEntity)
export class HomePageSubNavRepository extends Repository<HomePageSubNavEntity> {

    async createEntity(dto: BaseHomePageSubNavDto): Promise<HomePageSubNavEntity | Error> {
        try {
            const entity = this.create({
                title: dto.title,
                imageOne: dto.imageOne,
                imageTwo: dto.imageTwo,
                link: dto.link
            });
            await this.save(entity);
            return entity;
        } catch (e) {
            return e;
        }
    }

    async updateEntity(dto: UpdateHomePageSubNavDto, id: string): Promise<HomePageSubNavEntity | Error> {
        try {
            await this.update(id, {
                title: dto.title,
                imageOne: dto.imageOne,
                imageTwo: dto.imageTwo,
                link: dto.link
            });
            return await this.findOne(id);
        } catch (e) {
            return e;
        }
    }

    async findAll(): Promise<HomePageSubNavEntity[] | Error> {
        try {
            return await this.find({
                where: {
                    isDeleted: false
                }
            });
        } catch (e) {
            return e;
        }
    }

    async findOneEntity(id: string): Promise<HomePageSubNavEntity | Error> {
        try {
            return await this.findOne(id);
        } catch (e) {
            return e;
        }
    }

    async deleteEntity(id: string): Promise<boolean | Error> {
        try {
            const updateResult = await getConnection()
              .createQueryBuilder()
              .update(HomePageSubNavEntity)
              .set({ isDeleted: true })
              .where("id = :id", { id: id })
              .execute();
            if (updateResult.affected > 0) {
                return true;
            }
            return false;
        } catch (e) {
            return e;
        }
    }
}