import { HttpException, HttpStatus, Injectable } from "@nestjs/common";
import { InjectRepository } from "@nestjs/typeorm";
import { Constants } from "src/utils/constants";
import { ResponseUtils } from "src/utils/response.utils";
import { DoSpacesService } from "src/modules/SpacesModule/SpacesService/doSpacesService";
import { UploadedMulterFileI } from "src/modules/SpacesModule/SpacesService";
import { HomePageSubNavRepository } from "./home-page-sub-nav.repository";
import { BaseHomePageSubNavDto } from "./dto/base-home-page-sub-nav.dto";
import { HomePageSubNavEntity } from "./entities/home-page-sub-nav.entity";
import { UpdateHomePageSubNavDto } from "./dto/update-home-page-sub-nav.dto";

@Injectable()
export class HomePageSubNavService {
    constructor(
      @InjectRepository(HomePageSubNavRepository)
      private readonly repository: HomePageSubNavRepository,
      private readonly doSpaceService: DoSpacesService
    ) {
    }

    async create(dto: BaseHomePageSubNavDto, files: {
        imageOne: UploadedMulterFileI,
        imageTwo: UploadedMulterFileI,
    }): Promise<HomePageSubNavEntity | Error> {
        if (files && files.imageOne) {
            const imageOne: any = await this.doSpaceService.uploadFile(files.imageOne[0], "HomePageSubNavCms/");
            dto.imageOne = imageOne;
        }

        if (files && files.imageTwo) {
            const imageTwo: any = await this.doSpaceService.uploadFile(files.imageTwo[0], "HomePageSubNavCms/");
            dto.imageTwo = imageTwo;
        }
        const data = await this.repository.createEntity(dto);
        if (!data) {
            throw new HttpException(
              Constants.CREATE_FAILED,
              HttpStatus.BAD_REQUEST
            );
        }
        return ResponseUtils.successResponseHandler(201, "Data created successfully", "data", data);
    }

    async update(
      id: string,
      dto: UpdateHomePageSubNavDto,
      files: {
          imageOne: UploadedMulterFileI,
          imageTwo: UploadedMulterFileI,
      }
    ): Promise<HomePageSubNavEntity | Error> {
        const res: any = await this.repository.findOneEntity(id);
        if (!res) {
            throw new HttpException(
              Constants.NOT_FOUND,
              HttpStatus.NOT_FOUND
            );
        }
        if (files && files.imageOne) {
            const imageOne: any = await this.doSpaceService.uploadFile(files.imageOne[0], "HomePageSubNavCms/");
            dto.imageOne = imageOne;
        }

        if (files && files.imageTwo) {
            const imageTwo: any = await this.doSpaceService.uploadFile(files.imageTwo[0], "HomePageSubNavCms/");
            dto.imageTwo = imageTwo;
        }
        const data = await this.repository.updateEntity(dto, id);
        if (!data) {
            throw new HttpException(
              Constants.UPDATE_FAILED,
              HttpStatus.BAD_REQUEST
            );
        }
        return ResponseUtils.successResponseHandler(200, "Data updated successfully", "data", data);
    }

    async findAll(): Promise<HomePageSubNavEntity[] | Error> {
        const data = await this.repository.findAll();
        if (!data) {
            throw new HttpException(
              Constants.NOT_FOUND,
              HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(200, "Data fetched successfully", "data", data);
    }

    async findOne(id: string): Promise<HomePageSubNavEntity[] | Error> {
        const data = await this.repository.findOne({ id: id });
        if (!data) {
            throw new HttpException(
              Constants.NOT_FOUND,
              HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(200, "Data fetched successfully", "data", data);
    }

    async remove(id: string): Promise<boolean | Error> {
        const data = await this.repository.deleteEntity(id);
        if (!data) {
            throw new HttpException(
              Constants.NOT_FOUND,
              HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.buildDeletedData(data, 200, "Data deleted successfully");
    }
}