import { EntityRepository, Repository } from "typeorm";
import { OfferPopupCmsEntity } from "./entities/offer-popup-cms.entity";
import { UpdateOfferPopupCmsDto } from "./dto/update-offer-popup-cms.dto";

@EntityRepository(OfferPopupCmsEntity)
export class OfferPopupCmsRepository extends Repository<OfferPopupCmsEntity> {

    async createEntity(dto: UpdateOfferPopupCmsDto): Promise<OfferPopupCmsEntity | Error> {
        try {
            const entity = this.create({
                stateCode: dto.stateCode,
                stateName: dto.stateName,
                image: dto.image,
                title: dto.title,
                subTitle: dto.subTitle,
                bottomButtonText: dto.bottomButtonText,
                offerButtonText: dto.offerButtonText,
                offerButtonLink: dto.offerButtonLink,
                time: dto.time,
                expireTime: dto.expireTime,
                pages: dto.pages,
                isDefault: dto.isDefault
            });
            await this.save(entity);
            return entity;
        } catch (e) {
            return e;
        }
    }

    async updateEntity(dto: UpdateOfferPopupCmsDto, id: string): Promise<OfferPopupCmsEntity | Error> {
        try {
            await this.update(id, {
                stateCode: dto.stateCode,
                stateName: dto.stateName,
                image: dto.image,
                title: dto.title,
                subTitle: dto.subTitle,
                bottomButtonText: dto.bottomButtonText,
                offerButtonText: dto.offerButtonText,
                offerButtonLink: dto.offerButtonLink,
                time: dto.time,
                expireTime: dto.expireTime,
                pages: dto.pages,
                isDefault: dto.isDefault
            });
            return await this.findOne(id);
        } catch (e) {
            return e;
        }
    }

    async findAll(): Promise<OfferPopupCmsEntity[] | Error> {
        try {
            return await this.find({
                where: {
                    isDeleted: false
                }
            });
        } catch (e) {
            return e;
        }
    }
}