import { HttpException, HttpStatus, Injectable } from "@nestjs/common";
import { SectionThreeDto } from "./dto/section-three.dto";
import { SectionThreeRepository } from "./section-three.repository";
import { InjectRepository } from "@nestjs/typeorm";
import { DoSpacesService } from "src/modules/SpacesModule/SpacesService/doSpacesService";
import { UploadedMulterFileI } from "src/modules/SpacesModule/SpacesService";
import { SectionThree } from "./entities/section-three.entity";
import { Constants } from "src/utils/constants";
import { ResponseUtils } from "src/utils/response.utils";

@Injectable()
export class SectionThreeService {
    constructor(
      @InjectRepository(SectionThreeRepository)
      private readonly repository: SectionThreeRepository,
      private readonly doSpaceService: DoSpacesService
    ) {
    }

    async updateOrCreateData(
      dto: SectionThreeDto,
      files: {
          imageOne: UploadedMulterFileI,
          imageTwo: UploadedMulterFileI,
          imageThree: UploadedMulterFileI
      }
    ): Promise<SectionThree | Error> {
        if (files && files.imageOne) {
            const imageOne: any = await this.doSpaceService.uploadFile(files.imageOne[0], "SectionThreeCMS/");
            dto.imageOne = imageOne;
        }

        if (files && files.imageTwo) {
            const imageTwo: any = await this.doSpaceService.uploadFile(files.imageTwo[0], "SectionThreeCMS/");
            dto.imageTwo = imageTwo;
        }

        if (files && files.imageThree) {
            const imageThree: any = await this.doSpaceService.uploadFile(files.imageThree[0], "SectionThreeCMS/");
            dto.imageThree = imageThree;
        }

        const isExists = await this.repository.find({ isDeleted: false });
        if (Object.keys(isExists).length === 0) {
            const data = await this.repository.createEntity(dto);
            if (!data) {
                throw new HttpException(
                  Constants.NOT_FOUND,
                  HttpStatus.NOT_FOUND
                );
            }
            return ResponseUtils.successResponseHandler(201, "Data created successfully!", "data", data);
        } else {
            let id = isExists[0].id;
            const data = await this.repository.updateEntity(dto, id);
            if (!data) {
                throw new HttpException(
                  Constants.UPDATE_FAILED,
                  HttpStatus.BAD_REQUEST
                );
            }
            return ResponseUtils.successResponseHandler(200, "Data updated successfully!", "data", data);
        }
    }

    async findAll(): Promise<SectionThree[] | Error> {
        const data = await this.repository.findAll();
        if (!data) {
            throw new HttpException(Constants.NOT_FOUND, HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(200, "Data fetched successfully", "data", data);
    }
}
