import { HttpException, HttpStatus, Injectable } from "@nestjs/common";
import { InjectRepository } from "@nestjs/typeorm";
import { Constants } from "src/utils/constants";
import { ResponseUtils } from "src/utils/response.utils";
import { UploadedMulterFileI } from "../../SpacesModule/SpacesService";
import { DoSpacesService } from "../../SpacesModule/SpacesService/doSpacesService";
import { DeliveryPagesRepository } from "./delivery-pages.repository";
import { CreateDeliveryPagesDto } from "./dto/create-delivery-pages.dto";
import { UpdateDeliveryPagesDto } from "./dto/update-delivery-pages.dto";
import { DeliveryPages } from "./entities/delivery-pages.entity";
import { urlSlug } from "src/helper/common-helper";

@Injectable()
export class DeliveryPagesService {
    constructor(
      @InjectRepository(DeliveryPagesRepository)
      private readonly repository: DeliveryPagesRepository,
      private readonly doSpaceService: DoSpacesService
    ) {
    }

    async create(dto: CreateDeliveryPagesDto, files: {bannerBackgroundImage: UploadedMulterFileI}): Promise<DeliveryPages | Error> {
        if (files && files.bannerBackgroundImage) {
            const image: any = await this.doSpaceService.uploadFile(files.bannerBackgroundImage[0], "DeliveryPages/");
            dto.bannerBackgroundImage = image;
        }
        const splitedUrl = dto?.urlSlug?.split("delivery/");
        const splitedUrlSlug = urlSlug(splitedUrl?.[1])
        dto.urlSlug = splitedUrl?.[0] + "delivery/"+ splitedUrlSlug;

        const data = await this.repository.createEntity(dto);
        if (!data) {
            throw new HttpException(
              Constants.CREATE_FAILED,
              HttpStatus.BAD_REQUEST
            );
        }
        return ResponseUtils.successResponseHandler(201, "Data created successfully", "data", data);
    }

    async update(
      id: string,
      dto: UpdateDeliveryPagesDto,
      files: {
        bannerBackgroundImage: UploadedMulterFileI,
      }
    ): Promise<DeliveryPages | Error> {
        const res: any = await this.repository.findOneEntity(id);
        if (!res) {
            throw new HttpException(
              Constants.NOT_FOUND,
              HttpStatus.NOT_FOUND
            );
        }
        if (files && files.bannerBackgroundImage) {
            const image: any = await this.doSpaceService.uploadFile(files.bannerBackgroundImage[0], "DeliveryPages/");
            dto.bannerBackgroundImage = image;
        }
        const splitedUrl = dto?.urlSlug?.split("delivery/");
        const splitedUrlSlug = urlSlug(splitedUrl?.[1])
        dto.urlSlug = splitedUrl?.[0] + "delivery/"+ splitedUrlSlug;
        
        const data = await this.repository.updateEntity(id, dto);
        if (!data) {
            throw new HttpException(
              Constants.UPDATE_FAILED,
              HttpStatus.BAD_REQUEST
            );
        }
        return ResponseUtils.successResponseHandler(200, "Data updated successfully", "data", data);
    }

    async findAll(): Promise<DeliveryPages[] | Error> {
        const data = await this.repository.findByFilterQuery({});
        if (!data) {
            throw new HttpException(
              Constants.NOT_FOUND,
              HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(200, "Data fetched successfully", "data", data);
    }

    async findOne(id: string): Promise<DeliveryPages[] | Error> {
        const data = await this.repository.findByFilterQuery({ id: id });
        if (!data) {
            throw new HttpException(
              Constants.NOT_FOUND,
              HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(200, "Data fetched successfully", "data", data);
    }

    async remove(id: string): Promise<boolean | Error> {
        const data = await this.repository.deleteEntity(id);
        if (!data) {
            throw new HttpException(
              Constants.NOT_FOUND,
              HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.buildDeletedData(data, 200, "Data deleted successfully");
    }
}
