import { EntityRepository, Repository } from "typeorm";
import { UpdateCareerPageUiDto } from "./dto/update-career-page-ui.dto";
import { CareerPageUI } from "./entities/career-page-ui.entity";

@EntityRepository(CareerPageUI)
export class CareerPageUiRepository extends Repository<CareerPageUI> {

    async createEntity(dto: UpdateCareerPageUiDto): Promise<CareerPageUI | Error> {
        try {
            const entity = this.create({
                titleFontColor: dto.titleFontColor,
                subTitleFontColor: dto.subTitleFontColor,
                descriptionFontColor: dto.descriptionFontColor,
                buttonBackgroundColor: dto.buttonBackgroundColor,
                buttonFontColor: dto.buttonFontColor,
                backgroundColor: dto.backgroundColor
            });
            await this.save(entity);
            return entity;
        } catch (e) {
            return e;
        }
    }

    async updateEntity(dto: UpdateCareerPageUiDto, id: string): Promise<CareerPageUI | Error> {
        try {
            await this.update(id, {
                titleFontColor: dto.titleFontColor,
                subTitleFontColor: dto.subTitleFontColor,
                descriptionFontColor: dto.descriptionFontColor,
                buttonBackgroundColor: dto.buttonBackgroundColor,
                buttonFontColor: dto.buttonFontColor,
                backgroundColor: dto.backgroundColor
            });
            return await this.findOne(id);
        } catch (e) {
            return e;
        }
    }

    async findAll(): Promise<CareerPageUI[] | Error> {
        try {
            return await this.find({
                where: {
                    isDeleted: false
                }
            });
        } catch (e) {
            return e;
        }
    }
}