import { HttpException, HttpStatus, Injectable } from "@nestjs/common";
import { Constants } from "src/utils/constants";
import { StateWiseProductCategoryRepository } from "./state-wise-product-category.repository";
import { ResponseUtils } from "src/utils/response.utils";
import { DoSpacesService } from "src/modules/SpacesModule/SpacesService/doSpacesService";
import { ProductCategoryRepository } from "../product-category/product-category.repository";

@Injectable()
export class StateWiseProductCategoryService {
    constructor(
      private readonly repository: StateWiseProductCategoryRepository,
      private readonly doSpaceService: DoSpacesService,
      private readonly productCategoryRepo: ProductCategoryRepository
    ) {
    }

    async createStateWiseCategory(dto: any): Promise<any | Error> {
        if (dto.categories) {
            // console.log(dto.categories)
            for (const element of dto.categories) {
                const data = await this.repository.findOne({
                    categoryId: element.categoryId,
                    stateCode: element.stateCode,
                    isDeleted: false
                });
                if (!data) {
                    const createData = await this.repository.create({
                        stateCode: element.stateCode,
                        stateName: element.stateName,
                        categoryId: element.categoryId,
                        categoryName: element.categoryName
                    });
                    await this.repository.save(createData)
                    // console.log(createData);
                }
            }
        }

        return ResponseUtils.successResponseHandler(
          200,
          "Product state wise category data Created successfully",
          "data",
          {}
        );
    }

    async dragDropCategoryPosition(dto: any): Promise<any | Error> {
        if (dto.newOrderList) {
            for (const element of dto.newOrderList) {
                const data = await this.repository.update(element.id, {
                    position: element.position
                });
            }
        }

        return ResponseUtils.successResponseHandler(
          200,
          "Product state wise category position updated successfully",
          "data",
          {}
        );
    }

    async findAll(stateCode: string): Promise<any | Error> {
        const data = await this.repository.find({
            stateCode: stateCode,
            isDeleted: false
        });
        if (Object.keys(data).length === 0) {
            throw new HttpException(
              Constants.NOT_FOUND,
              HttpStatus.NOT_FOUND
            );
        }
        const uniqueCategoryWithSerial = data.map((item, index) => ({
            ...item,
            serial: index + 1
        }));
        // const orderedBrands = uniqueBrandsWithSerial.sort((a, b) => a.position - b.position);
        const orderedCategory = uniqueCategoryWithSerial.sort((a, b) => {
            if (a.position && b.position) {
                return a.position - b.position;
            } else if (a.position) {
                return -1; // Place objects with position first
            } else if (b.position) {
                return 1; // Place objects with position last
            }
            return a.serial - b.serial; // If no position, sort by serial
        });
        return ResponseUtils.successResponseHandler(200, "Product state wise category fetched successfully", "data", orderedCategory);
    }

    async update(id: string, dto: any): Promise<any|Error> {
        const data = await this.repository.findOne({
            id: id
        });
        if (!data) {
            throw new HttpException(
              Constants.NOT_FOUND,
              HttpStatus.NOT_FOUND
            );
        }
        const updateData = await this.repository.update(id, {
            isShown: dto.isShown
        });

        return ResponseUtils.successResponseHandler(200, "Product state wise category updated successfully", "data", data);
    }

    async makeDefault(dto: any): Promise<any | Error> {
        // Retrieve all entities
        const entities = await this.repository.find();

        // Update the isDefault property for each entity
        for (const entity of entities) {
            const updateData = await this.repository.update(entity.id, { isDefault: false });
        };

        // retrive data using state code
        const data = await this.repository.find({
            stateCode: dto.stateCode,
            isDeleted: false
        });

        for (const entity of data) {
            const updateData = await this.repository.update(entity.id, { isDefault: true });
        }
        ;

        const finalData = await this.repository.find({
            stateCode: dto.stateCode,
            isDeleted: false
        });

        return ResponseUtils.successResponseHandler(200, "Data updated successfully", "data", finalData);
    }

    async stateWiseAll(stateCode: string): Promise<any | Error> {
        let finalCategory = []
        const data: any = await this.repository.find({
            stateCode: stateCode,
            isShown: true,
            isDeleted: false
        });
        if (Object.keys(data).length === 0) {
            throw new HttpException(
              Constants.NOT_FOUND,
              HttpStatus.NOT_FOUND
            );
        }
        for(let i=0; i< Object.keys(data).length; i++) {
            const productCategory: any = await this.productCategoryRepo.findOne({
                id: data[i].categoryId,
                isDeleted: false
            });
            if(productCategory){
                productCategory.position = data[i].position;
                productCategory.serial = data[i].serial;
                finalCategory.push(productCategory);
            }
        }
        // const uniqueCategoryWithSerial = data.map((item, index) => ({
        //     ...item,
        //     serial: index + 1
        // }));
        // const orderedBrands = uniqueBrandsWithSerial.sort((a, b) => a.position - b.position);
        const orderedCategory = finalCategory.sort((a, b) => {
            if (a.position && b.position) {
                return a.position - b.position;
            } else if (a.position) {
                return -1; // Place objects with position first
            } else if (b.position) {
                return 1; // Place objects with position last
            }
            return a.serial - b.serial; // If no position, sort by serial
        });
        return ResponseUtils.successResponseHandler(200, "Product state wise category fetched successfully", "data", orderedCategory);
    }
}
