import {
    Body,
    Controller,
    Delete,
    Get,
    Param,
    Patch,
    Post,
    Query,
    UploadedFiles,
    UseGuards,
    UseInterceptors
} from "@nestjs/common";
import { Constants } from "src/utils/constants";
import { HomePageSectionOneService } from "./home-page-section-one.service";
import { CreateHomePageSectionOneDto } from "./dto/create-home-page-section-one.dto";
import { UpdateHomePageSectionOneDto } from "./dto/update-home-page-section-one.dto";
import { AdminAuthGuard } from "src/core/guards/admin-auth.guard";
import { FileFieldsInterceptor } from "@nestjs/platform-express";
import { UploadedMulterFileI } from "../SpacesModule/SpacesService";

@Controller({ path: "home-page-section-one", version: Constants.API_VERSION_1 })

export class HomePageSectionOneController {
    constructor(private readonly service: HomePageSectionOneService) {
    }

    // @Post("/make-default")
    // makeDefault(@Body() dto: any) {
    //     return this.service.makeDefault(dto);
    // }

    @Post()
    @UseGuards(AdminAuthGuard)
    @UseInterceptors(FileFieldsInterceptor(
      [
          { name: "image", maxCount: 1 }
      ]
    ))
    create(@Body() dto: CreateHomePageSectionOneDto,
           @UploadedFiles() files: {
               image: UploadedMulterFileI
           }) {
        return this.service.create(dto, files);
    }

    @Get()
    @UseGuards(AdminAuthGuard)
    // findAll(@Query("stateCode") stateCode: string)
    findAll() {
        return this.service.findAll();
    }

    @Get(":id")
    @UseGuards(AdminAuthGuard)
    findOne(@Param("id") id: string) {
        return this.service.findOne(id);
    }

    @Patch("/:id")
    @UseGuards(AdminAuthGuard)
    @UseInterceptors(FileFieldsInterceptor(
      [
          { name: "image", maxCount: 1 }
      ]
    ))
    update(
      @Param("id") id: string,
      @Body() dto: UpdateHomePageSectionOneDto,
      @UploadedFiles() files: {
          image: UploadedMulterFileI,
      }) {
        return this.service.update(id, dto, files);
    }

    @Delete(":id")
    @UseGuards(AdminAuthGuard)
    async remove(@Param("id") id: string) {
        return await this.service.remove(id);
    }
}
