import { HttpException, HttpStatus, Injectable } from "@nestjs/common";
import { ElasticsearchService } from "@nestjs/elasticsearch";
import { InjectRepository } from "@nestjs/typeorm";
import axios from "axios";
import { iHeartJaneAuthentication } from "src/helper/iheartjane-client-helper";
import { RetailerImageGalleryRepository } from "src/modules/retailer-image-gallery/retailer-image-gallery.repository";
import { Constants } from "src/utils/constants";
import { ResponseUtils } from "src/utils/response.utils";
import { UploadedMulterFileI } from "../../../SpacesModule/SpacesService";
import { DoSpacesService } from "../../../SpacesModule/SpacesService/doSpacesService";
import { CreateRetailerServiceIHeartJaneDto } from "./dto/create-retailer-service-iheartjane.dto";
import { SearchRetailerIHeartJaneDto } from "./dto/search-retailer-iheartjnae.dto";
import { RetailerServiceIHeartJane } from "./entities/retailer-service-iheartjane.entity";
import { RetailerServiceIHeartJaneRepository } from "./retailer-service-iheartjane.repository";

@Injectable()
export class RetailerServiceIHeartJaneService {
    constructor(
        @InjectRepository(RetailerServiceIHeartJaneRepository)
        private readonly repository: RetailerServiceIHeartJaneRepository,
        private readonly doSpaceService: DoSpacesService,
        private readonly esService: ElasticsearchService,
        private readonly retailerImageGalleryRepo: RetailerImageGalleryRepository
    ) {
    }

    async create(
        dto: CreateRetailerServiceIHeartJaneDto
    ): Promise<RetailerServiceIHeartJane | Error> {
        const data = await this.repository.createEntity(dto);
        if (!data) {
            throw new HttpException(
                Constants.CREATE_FAILED,
                HttpStatus.BAD_REQUEST
            );
        }
        return ResponseUtils.successResponseHandler(201, "Data created successfully", "data", data);
    }

    async findAll(): Promise<RetailerServiceIHeartJane[] | Error> {
        const data = await this.repository.findByFilterQuery({});
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        const finalData = [];
        for (const iterator of data) {
            const retailerimage = await this.retailerImageGalleryRepo.find({
                where: {
                    retailerId: iterator.retailerId,
                    isDeleted: false
                }
            });
            iterator.retailerImage = retailerimage;
            finalData.push(iterator);
        }
        return ResponseUtils.successResponseHandler(200, "Data fetched successfully", "data", finalData);
    }

    async findOne(id: string): Promise<RetailerServiceIHeartJane[] | Error> {
        const data = await this.repository.findByFilterQuery({ productId: id });
        const reviewCount = Object.keys(data).length != 0 ? Object.keys(data).length : 0;
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        let ratingAvarage = 0;
        if (Object.keys(data).length > 0) {
            let count = 0;
            for (let i = 0; i < reviewCount; i++) {
                count += parseInt(data[i].rating);
            }
            ratingAvarage = Math.round(count / reviewCount);
        }

        const response = {
            data: data,
            reviewCount: reviewCount,
            ratingAvarage: ratingAvarage
        };
        return ResponseUtils.successResponseHandler(200, "Data fetched successfully", "data", response);
    }

    async remove(id: string): Promise<boolean | Error> {
        const data = await this.repository.deleteEntity(id);
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.buildDeletedData(data, 200, "Data deleted successfully");
    }

    async update(
        id: string,
        dto: any,
        files: {
            retailerImage?: UploadedMulterFileI,
            featuredImage?: UploadedMulterFileI
        }
    ): Promise<any | Error> {
        const findData = await this.repository.findOne({ where: { id: id } });
        if (!findData) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.BAD_REQUEST
            );
        }
        if (files && files.retailerImage) {
            const retailerFile = files['retailerImage'];

            Array(retailerFile).forEach(async (element: any) => {
                element.forEach(async (element: any, index: number) => {
                    const retailerImage: any = await this.doSpaceService.uploadFile(element, "DutchiePlusRetailerServiceIHeartJane/");
                    const retailerImageDto = {
                        retailerId: findData.retailerId,
                        retailerName: findData.retailerName,
                        image: retailerImage
                    };
                    await this.retailerImageGalleryRepo.createEntity(retailerImageDto);
                });
            });

        }
        if (dto.deletedImage && dto.deletedImage.length > 0) {
            await this.retailerImageGalleryRepo.deleteImage(dto.deletedImage);
        }

        let image: any;
        if (files && files.featuredImage) {
            image = await this.doSpaceService.uploadFile(files.featuredImage[0], "DutchiePlusRetailerServiceIHeartJane/");
        }

        if (dto.featuredImage) {
            image = dto.featuredImage;
        }

        await this.repository.update(id, {
            featuredImage: image,
            about: dto.about,
            todaysSpecial: dto.todaysSpecial,
            email: dto.email,
            jarsRewards: dto.jarsRewards,
            referAFriend: dto.referAFriend,
            pageTitle: dto.pageTitle,
            schema: dto.schema,
            canonical: dto.canonical,
        });
        const data = await this.repository.findOne({ where: { id: id } });
        if (!data) {
            throw new HttpException(
                Constants.UPDATE_FAILED,
                HttpStatus.BAD_REQUEST
            );
        }
        return ResponseUtils.successResponseHandler(200, "Data Updated successfully", "data", data);
    }

    async updateAlpineIq(id: string, dto: any): Promise<any | Error> {

        const FedexCutoffTime: any = await this.repository.findOneEntity(id);
        if (!FedexCutoffTime) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }

        const updateData = await this.repository.update(id, {
            alpineIqApiKey: dto.alpineIqApiKey,
            alpineIqUID: dto.alpineIqUID
        });
        const data = await this.repository.findOne(id);
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }

        return ResponseUtils.successResponseHandler(
            200,
            "Data Updated Successful!",
            "data",
            data);
    }

    public async indexIsExists(index: string) {
        return await this.esService.indices.exists({ index });
    }

    public async deleteIndex(index: string) {
        const isExists = await this.indexIsExists(index);
        if (isExists) {
            await this.esService.indices.delete({ index });
        }
    }

    public async createIndex(index: string, body: any) {
        const isExists = await this.indexIsExists(index);
        if (!isExists) {
            const data = await this.esService.indices.create({ index, body });
            return data.acknowledged;
        }
        return false;
    }

    public async indexData(index: string, body: any) {
        return await this.esService.index({ index, body });
    }

    private async getRetailerData() {
        try {
            const authentication: any = await iHeartJaneAuthentication();
            if (authentication.access_token) {
                const apiUrl = `${process.env.IHEARTJANE_API_HOST}/api/roots/store_operations_api/v1/stores`;
                const config = {
                    headers: {
                        "Content-Type": "application/json",
                        "Authorization": `Bearer ${authentication.access_token}`
                    }
                };
                const response = await axios.get(apiUrl, config);
                if (response.status === 200) {
                    return response.data;
                }
            }
        } catch (error) {
            console.log(error);
            return error;
        }
    }

    public async indexBulkData(indexValue: string) {
        console.log(">>>Start Bulk Insert Retailer Service<<<", new Date());
        // const res: any = await this.findAll();
        const retailers = await this.getRetailerData();
        //console.log(retailers?.data?.stores);

        await this.deleteIndex(indexValue);

        const body = {
            settings: {
                analysis: {
                    filter: {
                        autocomplete_filter: {
                            type: "edge_ngram",
                            min_gram: 1,
                            max_gram: 15
                        }
                    },
                    analyzer: {
                        autocomplete: {
                            type: "custom",
                            tokenizer: "standard",
                            filter: ["lowercase", "autocomplete_filter"]
                        }
                    }
                }
            },

            mappings: {
                properties: {
                    retailerId: {
                        type: "text",
                        fields: {
                            keyword: {
                                type: "keyword",
                                ignore_above: 256
                            }
                        }
                    },
                    state: {
                        type: "text",
                        fields: {
                            keyword: {
                                type: "keyword",
                                ignore_above: 256
                            }
                        }
                    },
                    retailerName: {
                        type: "text",
                        analyzer: "autocomplete",
                        search_analyzer: "standard",
                        fields: {
                            keyword: {
                                type: "keyword",
                                ignore_above: 256
                            }
                        }
                    },
                    searchKey: {
                        type: "text",
                        analyzer: "autocomplete",
                        search_analyzer: "standard",
                        fields: {
                            keyword: {
                                type: "keyword",
                                ignore_above: 512
                            }
                        }
                    }
                }
            }
        };

        await this.createIndex(indexValue, body);
        const operations = retailers?.data?.stores.flatMap((doc) => [
            { index: { _index: indexValue } },
            // {
            //     update: {
            //         _index: indexValue,
            //         _id: doc.id, // Assuming retaileId is a unique identifier
            //     }
            // },
            // {
            //     doc_as_upsert: true, doc: 
            {
                retailerId: doc.id,
                retailerName: doc.name,
                state: "AZ",
                //doc.full_address.split(",")[2],
                searchKey: `${doc.name} ${doc.full_address.split(",")[3]}`
            }
            //}
        ]);
        await this.esService.bulk({ refresh: true, operations });
        console.log(">>>End Bulk Insert RetailerServiceIHeartJane<<<", new Date());

        return ResponseUtils.successResponseHandler(200, "Elastic DataBase RetailerServiceIHeartJane Update Successfully.");
    }

    public async search(index: string, searchDto: SearchRetailerIHeartJaneDto) {
        const { search, limit, state } = searchDto;

        const exactsMatch = [];
        if (state && state != "") {
            const stateMatch = {
                match_phrase: {
                    state: state.toLowerCase()
                }
            };
            exactsMatch.push(stateMatch);
        }

        const result = await this.esService.search<any>({
            index,
            body: {
                query: {
                    bool: {
                        must: [
                            ...exactsMatch,
                            {
                                match: {
                                    searchKey: {
                                        query: search,
                                        fuzziness: "AUTO",
                                        fuzzy_transpositions: false,
                                        prefix_length: 2
                                    }
                                }
                            }
                        ]
                    }
                }
                /*
                highlight: {
                    fields: {
                        retailerName: {
                            pre_tags: ["<b>"],
                            post_tags: ["</b>"]
                        }
                    }
                }
                */
            },
            size: limit && limit > 0 ? limit : 50
        });
        const hits = result.hits.hits;
        const pageCount: any = result.hits.total;

        const retailers = hits.map((item: any) => {
            return {
                ...item._source
                // highlight: item?.highlight?.retailerName?.[0]
            };
        });
        const data = { retailers, total: (pageCount?.value ?? pageCount) };

        if (!data) {
            throw new HttpException(Constants.NOT_FOUND, HttpStatus.NOT_FOUND);
        }
        return ResponseUtils.successResponseHandler(
            200,
            "Retailers list fetched successfully",
            "data",
            data
        );
    }

    async checkRetailerDifference(data: any): Promise<any | Error> {
        try {
            const retailerServiceIHeartJane = await this.repository.findByFilterQuery({});

            if (Object.keys(retailerServiceIHeartJane).length > 0) {
                const existingRetailerIds = retailerServiceIHeartJane.map(x => x.retailerId);
                const difference = data?.data?.stores?.filter(obj => !existingRetailerIds.includes(obj.id));
                console.log("Retailer Difference =", difference);

                const dtoObjects = difference.map(obj => ({
                    retailerId: obj.id,
                    retailerName: obj.name,
                    full_address: obj.full_address,
                    delivery: obj.delivery,
                    description: obj.description,
                    google_map_url: obj.google_map_url,
                    hours: JSON.stringify(obj.hours),
                    lat: obj.lat,
                    long: obj.long,
                    medical: obj.medical,
                    phone: obj.phone,
                    photo: obj.photo,
                    pickup: obj.pickup,
                    recreational: obj.recreational,
                    time_zone_identifier: obj.time_zone_identifier,
                    url_slug: obj.url_slug
                }));

                await this.repository.createEntities(dtoObjects);
            } else {
                const dtoObjects = data?.data?.stores?.map(obj => ({
                    retailerId: obj.id,
                    retailerName: obj.name,
                    full_address: obj.full_address,
                    delivery: obj.delivery,
                    description: obj.description,
                    google_map_url: obj.google_map_url,
                    hours: JSON.stringify(obj.hours),
                    lat: obj.lat,
                    long: obj.long,
                    medical: obj.medical,
                    phone: obj.phone,
                    photo: obj.photo,
                    pickup: obj.pickup,
                    recreational: obj.recreational,
                    time_zone_identifier: obj.time_zone_identifier,
                    url_slug: obj.url_slug
                }));

                await this.repository.createEntities(dtoObjects);
            }

            return ResponseUtils.successResponseHandler(
                200,
                "Retailers Sync successfully",
                "data",
                {}
            );

        } catch (e) {
            // console.log(e);
            return e;
        }
    }

    async createUpdateRetailer() {
        try {
            const authentication: any = await iHeartJaneAuthentication();
            if (authentication.access_token) {
                const apiUrl = `${process.env.IHEARTJANE_API_HOST}/api/roots/store_operations_api/v1/stores`;
                const config = {
                    headers: {
                        "Content-Type": "application/json",
                        "Authorization": `Bearer ${authentication.access_token}`
                    }
                };
                const response = await axios.get(apiUrl, config);
                if (response.status === 200) {
                    // store retailers in local storage
                    const responseData = await this.checkRetailerDifference(response.data);
                    return responseData;
                }
            }
        } catch (e) {
            // console.log(e);
            return e;
        }
    }
}
