import { MailerService } from "@nestjs-modules/mailer";
import { HttpException, HttpStatus, Injectable } from "@nestjs/common";
import { Constants } from "../../utils/constants";
import { AdminNotifyDto } from "./dto/admin-notify.dto";
import { CreateAdminMailSenderDto } from "./dto/create-admin-mail-sender.dto";
import { CreateListingStatusDto } from "./dto/create-listing-status.dto";
import { CreateMailSenderDto } from "./dto/create-mail-sender.dto";
import { CreateNewListingDto } from "./dto/create-new-listing.dto";
import { CreateProductHintMailDto } from "./dto/create-product-hint-mail.dto";
import { ResetPasswordMailDto } from "./dto/create-reset-password-mail.dto";
import { CreateFranchiseMailSenderDto } from "./dto/franchise-mail-sender.dto";
import { OrderPlacedDto } from "./dto/order-placed-mail.dto";
import { OrderStatusChangeDto } from "./dto/order-status-change-mail.dto";
import { ContactMessageMailDto } from "./dto/create-contact-message-mail.dto";

@Injectable()
export class MailSenderService {
    constructor(private readonly mailerService: MailerService) {
    }

    async send(dto: CreateMailSenderDto) {
        console.log("Init mail");
        const { to, from, subject, username, password } = dto;
        const result = await this.mailerService
          .sendMail({
              to: to, // list of receivers
              bcc: [], // list of receivers
              from: from, // sender address
              subject: subject,
              template: "./templates/welcomeCustomerByAdmin", // The `.pug` or `.hbs` extension is appended automatically.
              context: {
                  // Data to be sent to template engine.
                  username: username,
                  password: password
              } // HTML body content
          })
          .catch((err) => {
              console.log(err);
              throw new HttpException(
                Constants.GENERIC_ERROR,
                HttpStatus.INTERNAL_SERVER_ERROR
              );
          });
        console.log("complete", result);
        return {
            message: "mail sent to " + to
        };
    }

    async accocuntCreateAdminNotify(dto: CreateAdminMailSenderDto) {
        const { to, from, subject, name, email } = dto;
        const result = await this.mailerService
          .sendMail({
              to: to, // list of receivers
              bcc: [], // list of receivers
              from: from, // sender address
              subject: subject,
              template: "./templates/account-create-admin", // The `.pug` or `.hbs` extension is appended automatically.
              context: {
                  // Data to be sent to template engine.
                  name: name,
                  email: email
              } // HTML body content
          })
          .catch((err) => {
              console.log(err);
              throw new HttpException(
                Constants.GENERIC_ERROR,
                HttpStatus.INTERNAL_SERVER_ERROR
              );
          });
        console.log("complete" + result);
        return {
            message: "mail sent to " + to
        };
    }

    async accocuntCreateByAdminCustomerNotify(dto: CreateMailSenderDto) {
        const { to, from, subject, username, password } = dto;
        const result = await this.mailerService
          .sendMail({
              to: to, // list of receivers
              bcc: [], // list of receivers
              from: from, // sender address
              subject: subject,
              template: "./templates/welcomeCustomerByAdminNew", // The `.pug` or `.hbs` extension is appended automatically.
              context: {
                  // Data to be sent to template engine.
                  username: username,
                  password: password
              } // HTML body content
          })
          .catch((err) => {
              console.log(err);
              throw new HttpException(
                Constants.GENERIC_ERROR,
                HttpStatus.INTERNAL_SERVER_ERROR
              );
          });
        console.log("complete", result);
        return {
            message: "mail sent to " + to
        };
    }

    async accocuntCreateCustomerNotify(dto: CreateMailSenderDto) {
        const { to, from, subject, username, password } = dto;
        const result = await this.mailerService
          .sendMail({
              to: to, // list of receivers
              bcc: [], // list of receivers
              from: from, // sender address
              subject: subject,
              template: "./templates/welcomeCustomerNew", // The `.pug` or `.hbs` extension is appended automatically.
              context: {
                  // Data to be sent to template engine.
                  username: username,
                  password: password
              } // HTML body content
          })
          .catch((err) => {
              console.log(err);
              throw new HttpException(
                Constants.GENERIC_ERROR,
                HttpStatus.INTERNAL_SERVER_ERROR
              );
          });
        console.log("complete", result);
        return {
            message: "mail sent to " + to
        };
    }

    async productHintMail(dto: CreateProductHintMailDto) {
        const {
            to,
            from,
            subject,
            senderName,
            message,
            productName,
            featuredImage,
            price,
            product
        } = dto;
        const result = await this.mailerService
          .sendMail({
              to: to,
              bcc: [],
              from: from,
              subject: subject,
              template: "./templates/product-hint-email",
              context: {
                  senderName: senderName,
                  message: message,
                  productName: productName,
                  featuredImage: featuredImage,
                  price: price,
                  product: product
              }
          })
          .catch((err) => {
              console.log(err);
              throw new HttpException(
                Constants.GENERIC_ERROR,
                HttpStatus.INTERNAL_SERVER_ERROR
              );
          });
        return {
            message: "Mail sent to " + to
        };
    }

    async franchiseCreateCustomerNotify(dto: CreateFranchiseMailSenderDto) {
        const {
            to,
            from,
            subject,
            firstName,
            lastName,
            franchiseName,
            phone,
            email,
            state,
            city,
            zipCode,
            country,
            notes
        } = dto;
        const result = await this.mailerService
          .sendMail({
              to: to, // list of receivers
              bcc: [], // list of receivers
              from: from, // sender address
              subject: subject,
              template: "./templates/create_franchise_customer_notify", // The `.pug` or `.hbs` extension is appended automatically.
              context: {
                  // Data to be sent to template engine.
                  firstName: firstName,
                  lastName: lastName,
                  franchiseName: franchiseName,
                  phone: phone,
                  email: email,
                  state: state,
                  city: city,
                  zipCode: zipCode,
                  country: country,
                  notes: notes
              } // HTML body content
          })
          .catch((err) => {
              console.log(err);
              throw new HttpException(
                Constants.GENERIC_ERROR,
                HttpStatus.INTERNAL_SERVER_ERROR
              );
          });
        console.log("complete" + result);
        return {
            message: "mail sent to " + to
        };
    }

    async franchiseCreateAdminNotify(dto: CreateFranchiseMailSenderDto) {
        const {
            to,
            from,
            subject,
            firstName,
            lastName,
            franchiseName,
            phone,
            email,
            state,
            city,
            zipCode,
            country,
            notes
        } = dto;
        const result = await this.mailerService
          .sendMail({
              to: to, // list of receivers
              bcc: [], // list of receivers
              from: from, // sender address
              subject: subject,
              template: "./templates/create_franchise_admin_notify", // The `.pug` or `.hbs` extension is appended automatically.
              context: {
                  // Data to be sent to template engine.
                  firstName: firstName,
                  lastName: lastName,
                  franchiseName: franchiseName,
                  phone: phone,
                  email: email,
                  state: state,
                  city: city,
                  zipCode: zipCode,
                  country: country,
                  notes: notes
              } // HTML body content
          })
          .catch((err) => {
              console.log(err);
              throw new HttpException(
                Constants.GENERIC_ERROR,
                HttpStatus.INTERNAL_SERVER_ERROR
              );
          });
        console.log("complete" + result);
        return {
            message: "mail sent to " + to
        };
    }

    async userApprovalNotify(dto: CreateMailSenderDto) {
        const { to, from, subject, username } = dto;
        const result = await this.mailerService
          .sendMail({
              to: to, // list of receivers\
              bcc: [], // list of receivers
              from: from, // sender address
              subject: subject,
              template: "./templates/approval-notify", // The `.pug` or `.hbs` extension is appended automatically.
              context: {
                  // Data to be sent to template engine.
                  username: username
              } // HTML body content
          })
          .catch((err) => {
              console.log(err);
              throw new HttpException(
                Constants.GENERIC_ERROR,
                HttpStatus.INTERNAL_SERVER_ERROR
              );
          });
        console.log("complete" + result);
        return {
            message: "mail sent to " + to
        };
    }

    async adminNotify(dto: AdminNotifyDto) {
        const { to, from, subject, name, email } = dto;
        const result = await this.mailerService
          .sendMail({
              to: to, // list of receivers
              bcc: [], // list of receivers
              from: from, // sender address
              subject: subject,
              template: "./templates/admin-notify", // The `.pug` or `.hbs` extension is appended automatically.
              context: {
                  // Data to be sent to template engine.
                  name: name,
                  email: email
              } // HTML body content
          })
          .catch((err) => {
              console.log(err);
              throw new HttpException(
                Constants.GENERIC_ERROR,
                HttpStatus.INTERNAL_SERVER_ERROR
              );
          });
        console.log("complete" + result);
        return {
            message: "mail sent to " + to
        };
    }

    async sendListingNotification(dto: CreateListingStatusDto) {
        const { to, from, subject, listingOwner, listingStatus, listingID } = dto;
        const result = await this.mailerService
          .sendMail({
              to: to, // list of receivers
              bcc: [], // list of receivers
              from: from, // sender address
              subject: subject,
              template: "./templates/listingnotification", // The `.pug` or `.hbs` extension is appended automatically.
              context: {
                  // Data to be sent to template engine.
                  listingOwner: listingOwner,
                  listingStatus: listingStatus,
                  listingID: listingID
              } // HTML body content
          })
          .catch((err) => {
              console.log(err);
              throw new HttpException(
                Constants.GENERIC_ERROR,
                HttpStatus.INTERNAL_SERVER_ERROR
              );
          });
        console.log("complete" + result);
        console.log("complete" + to);
        return {
            message: "mail sent to " + to
        };
    }

    async sendNewListingNotification(dto: CreateNewListingDto) {
        const { to, from, subject, listingOwner, listingStatus, listingID } = dto;
        const result = await this.mailerService
          .sendMail({
              to: to, // list of receivers
              bcc: [], // list of receivers
              from: from, // sender address
              subject: subject,
              template: "./templates/newlistingnotification", // The `.pug` or `.hbs` extension is appended automatically.
              context: {
                  // Data to be sent to template engine.
                  listingOwner: listingOwner,
                  listingStatus: listingStatus,
                  listingID: listingID
              } // HTML body content
          })
          .catch((err) => {
              console.log(err);
              throw new HttpException(
                Constants.GENERIC_ERROR,
                HttpStatus.INTERNAL_SERVER_ERROR
              );
          });
        console.log("complete" + result);
        console.log("complete" + to);
        return {
            message: "mail sent to " + to
        };
    }

    async sendPasswordResetRequestMail(dto: ResetPasswordMailDto) {
        const { to, from, subject, username, token } = dto;
        const result = await this.mailerService
          .sendMail({
              to: to, // list of receivers
              bcc: [], // list of receivers
              from: from, // sender address
              subject: subject,
              template: "./templates/password-reset-request-mail", // The `.pug` or `.hbs` extension is appended automatically.
              context: {
                  // Data to be sent to template engine.
                  username: username,
                  token: token
              } // HTML body content
          })
          .catch((err) => {
              console.log(err);
              throw new HttpException(
                Constants.GENERIC_ERROR,
                HttpStatus.INTERNAL_SERVER_ERROR
              );
          });
        console.log("complete" + result);
        console.log("complete" + to);
        return {
            message: "mail sent to " + to
        };
    }

    async sendContactMessageRequestMail(dto: ContactMessageMailDto) {
        const { to, from, subject, contactReason, name, email, message } = dto;
        const result = await this.mailerService
            .sendMail({
                to: to, // list of receivers
                bcc: [], // list of receivers
                from: from, // sender address
                subject: subject,
                template: "./templates/contact-message-request-mail", // The `.pug` or `.hbs` extension is appended automatically.
                context: {
                    // Data to be sent to template engine.
                    subject: subject,
                    contactReason: contactReason,
                    name: name,
                    email: email,
                    message: message
                } // HTML body content
            })
            .catch((err) => {
                console.log(err);
                throw new HttpException(
                    Constants.GENERIC_ERROR,
                    HttpStatus.INTERNAL_SERVER_ERROR
                );
            });
        console.log("complete" + result);
        console.log("complete" + to);
        return {
            message: "mail sent to " + to
        };
    }

    async sendPlaceOrderNotification(dto: OrderPlacedDto) {
        console.log("dro->>", dto);
        const {
            to,
            from,
            subject,
            customerName,
            orderData,
            orderDate,
            shippingAddress,
            totalAmount,
            contactInfo,
            items
        } = dto;
        const result = await this.mailerService
          .sendMail({
              to: to,
              bcc: [],
              from: from,
              subject: subject,
              template: "./templates/order-placed-email",
              context: {
                  customerName: customerName,
                  orderData: orderData,
                  orderDate: orderDate,
                  shippingAddress: shippingAddress,
                  totalAmount: totalAmount,
                  contactInfo: contactInfo,
                  orderItems: items
              }
          })
          .catch((err) => {
              console.log(err);
              throw new HttpException(
                Constants.GENERIC_ERROR,
                HttpStatus.INTERNAL_SERVER_ERROR
              );
          });
        return {
            message: "Mail sent to " + to
        };
    }

    async sendOrderStatusChangeNotification(dto: OrderStatusChangeDto) {
        console.log("dro->>", dto);
        const {
            to,
            from,
            subject,
            customerName,
            orderData,
            orderDate,
            shippingAddress,
            totalAmount,
            contactInfo
        } = dto;
        const result = await this.mailerService
          .sendMail({
              to: to,
              bcc: [],
              from: from,
              subject: subject,
              template: "./templates/order-shipped-email",
              context: {
                  customerName: customerName,
                  orderData: orderData,
                  orderDate: orderDate,
                  shippingAddress: shippingAddress,
                  totalAmount: totalAmount,
                  contactInfo: contactInfo
              }
          })
          .catch((err) => {
              console.log(err);
              throw new HttpException(
                Constants.GENERIC_ERROR,
                HttpStatus.INTERNAL_SERVER_ERROR
              );
          });
        return {
            message: "Mail sent to " + to
        };
    }
}
