import { HttpException, HttpStatus, Injectable } from "@nestjs/common";
import { InjectRepository } from "@nestjs/typeorm";
import { Constants } from "src/utils/constants";
import { ResponseUtils } from "src/utils/response.utils";
import { StayEnlightenedRepository } from "./stay-enlightened.repository";
import { CreateStayEnlightenedDto } from "./dto/create-stay-enlightened.dto";
import { UpdateStayEnlightenedDto } from "./dto/update-stay-enlightened.dto";
import { StayEnlightened } from "./entities/stay-enlightened.entity";

@Injectable()
export class StayEnlightenedService {
    constructor(
      @InjectRepository(StayEnlightenedRepository)
      private readonly repository: StayEnlightenedRepository
    ) {
    }

    async create(dto: CreateStayEnlightenedDto): Promise<StayEnlightened | Error> {
      const isEmailExists = await this.repository.findByEmailAddress(dto.email);
      if (isEmailExists) {
        throw new HttpException("Email Already Exists", HttpStatus.BAD_REQUEST);
      }

      const data = await this.repository.createEntity(dto);
      if (!data) {
          throw new HttpException(
            Constants.CREATE_FAILED,
            HttpStatus.BAD_REQUEST
          );
      }
      
      return ResponseUtils.successResponseHandler(201, "Data saved successfully", "data", data);
    }

    async update(id: string, dto: UpdateStayEnlightenedDto): Promise<StayEnlightened | Error> {
        const res: any = await this.repository.findOneEntity(id);
        if (!res) {
            throw new HttpException(
              Constants.NOT_FOUND,
              HttpStatus.NOT_FOUND
            );
        }
        const data = await this.repository.updateEntity(id, dto);
        if (!data) {
            throw new HttpException(
              Constants.UPDATE_FAILED,
              HttpStatus.BAD_REQUEST
            );
        }
        return ResponseUtils.successResponseHandler(200, "Data updated successfully", "data", data);
    }

    async findAll(): Promise<StayEnlightened[] | Error> {
        const data = await this.repository.findByFilterQuery({});
        if (!data) {
            throw new HttpException(
              Constants.NOT_FOUND,
              HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(200, "Data fetched successfully", "data", data);
    }

    async findOne(id: string): Promise<StayEnlightened[] | Error> {
        const data = await this.repository.findByFilterQuery({ id: id });
        if (!data) {
            throw new HttpException(
              Constants.NOT_FOUND,
              HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(200, "Data fetched successfully", "data", data);
    }

    async remove(id: string): Promise<boolean | Error> {
        const data = await this.repository.deleteEntity(id);
        if (!data) {
            throw new HttpException(
              Constants.NOT_FOUND,
              HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.buildDeletedData(data, 200, "Data deleted successfully");
    }
}
