import { Body, Controller, Delete, Get, Param, Patch, Post, Query, UseGuards } from "@nestjs/common";
import { Constants } from "src/utils/constants";
import { WeeklyAdTagsListService } from "./weekly-ad-tags-list.service";
import { CreateWeeklyAdTagsListDto } from "./dto/create-weekly-ad-tags-list.dto";
import { UpdateWeeklyAdTagsListDto } from "./dto/update-weekly-ad-tags-list.dto";
import { AdminAuthGuard } from "src/core/guards/admin-auth.guard";

@Controller({ path: "weekly-ad-tags-list", version: Constants.API_VERSION_1 })

export class WeeklyAdTagsListController {
    constructor(private readonly service: WeeklyAdTagsListService) {
    }

    @Post()
    @UseGuards(AdminAuthGuard)
    create(@Body() dto: CreateWeeklyAdTagsListDto) {
        return this.service.create(dto);
    }

    @Get()
    @UseGuards(AdminAuthGuard)
    findAll() {
        return this.service.findAll();
    }

    @Get("/filter")
    filterWeeklyAdByTag(
        @Query("tagId") tagId?: string,
        @Query("stateCode") stateCode?: string,
    ) {
        return this.service.filterWeeklyAdByTag(tagId, stateCode);
    }

    @Get(":id")
    @UseGuards(AdminAuthGuard)
    findOne(@Param("id") id: string) {
        return this.service.findOne(id);
    }

    @Patch("/:id")
    @UseGuards(AdminAuthGuard)
    update(@Param("id") id: string, @Body() dto: UpdateWeeklyAdTagsListDto) {
        return this.service.update(id, dto);
    }

    @Delete(":id")
    @UseGuards(AdminAuthGuard)
    async remove(@Param("id") id: string) {
        return await this.service.remove(id);
    }
}
