import { EntityRepository, getConnection, Repository } from "typeorm";
import { CreateWeeklyAdTagsDto } from "./dto/create-weekly-ad-tags.dto";
import { UpdateWeeklyAdTagsDto } from "./dto/update-weekly-ad-tags.dto";
import { WeeklyAdTags } from "./entities/weekly-ad-tags.entity";

@EntityRepository(WeeklyAdTags)
export class WeeklyAdTagsRepository extends Repository<WeeklyAdTags> {
    async createEntity(dto: CreateWeeklyAdTagsDto): Promise<WeeklyAdTags | Error> {
        try {
            const {
                tagName
            } = dto;
            const entity = this.create({
                tagName
            });
            await this.save(entity);
            return entity;
        } catch (e) {
            return e;
        }
    }

    async findOneEntity(id: string): Promise<WeeklyAdTags | Error> {
        try {
            return await this.findOne(id);
        } catch (e) {
            return e;
        }
    }

    async updateEntity(id: string, dto: UpdateWeeklyAdTagsDto): Promise<WeeklyAdTags | Error> {
        try {
            await this.update(id, {
                tagName: dto.tagName
            });
            return await this.findOne(id);
        } catch (e) {
            return e;
        }
    }

    async findByFilterQuery(query: any): Promise<WeeklyAdTags[] | Error> {
        try {
            return await this.find({
                where: { ...query, isDeleted: false }
            });
        } catch (e) {
            return e;
        }
    }

    async deleteEntity(id: string): Promise<boolean | Error> {
        try {
            const updateResult = await getConnection()
              .createQueryBuilder()
              .update(WeeklyAdTags)
              .set({ isDeleted: true })
              .where("id = :id", { id: id })
              .execute();
            if (updateResult.affected > 0) {
                return true;
            }
            return false;
        } catch (e) {
            return e;
        }
    }
}